/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.exportimport.util.comparator;

import com.liferay.exportimport.kernel.model.ExportImportConfiguration;
import com.liferay.portal.kernel.util.DateUtil;
import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.kernel.util.StringUtil;

/**
 * @author Daniel Kocsis
 */
public class ExportImportConfigurationNameComparator
	extends OrderByComparator<ExportImportConfiguration> {

	public static final String ORDER_BY_ASC = "name ASC, createDate DESC";

	public static final String ORDER_BY_DESC = "name DESC, createDate ASC";

	public static final String[] ORDER_BY_FIELDS = {"name", "createDate"};

	public ExportImportConfigurationNameComparator() {
		this(false);
	}

	public ExportImportConfigurationNameComparator(boolean ascending) {
		_ascending = ascending;
	}

	@Override
	public int compare(
		ExportImportConfiguration configuration1,
		ExportImportConfiguration configuration2) {

		String name1 = StringUtil.toLowerCase(configuration1.getName());
		String name2 = StringUtil.toLowerCase(configuration2.getName());

		int value = name1.compareTo(name2);

		if (value == 0) {
			value = DateUtil.compareTo(
				configuration1.getCreateDate(), configuration2.getCreateDate());
		}

		if (_ascending) {
			return value;
		}

		return -value;
	}

	@Override
	public String getOrderBy() {
		if (_ascending) {
			return ORDER_BY_ASC;
		}

		return ORDER_BY_DESC;
	}

	@Override
	public String[] getOrderByFields() {
		return ORDER_BY_FIELDS;
	}

	@Override
	public boolean isAscending() {
		return _ascending;
	}

	private final boolean _ascending;

}